#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
"""Gripe: convert json to jsonb

The jsonb type stands for "JSON binary" and has the following advantages:

- The data is stored compressed
- There are more sophisticated operators for querying, accessing, and modifying JSON in JSONB format

The type has the following disadvantages:

- Key/values are not stored "in order"
- Certain erroneous JSON values are no longer permitted in jsonb format (such as defining the same key twice)

In general the disadvantages are outweighed by the advantages, hence the migration.

Revision ID: 5255a60a2697
Revises: 702a4a776f9b
Create Date: 2022-01-24 13:29:23.207255

"""
import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = "5255a60a2697"
down_revision = "702a4a776f9b"
branch_labels = None
depends_on = None


def upgrade():
    op.execute("DROP VIEW download_metrics")
    op.execute("ALTER TABLE capability_state_machines ALTER COLUMN associated_workflows TYPE jsonb")
    op.execute(
        "ALTER TABLE capability_versions ALTER COLUMN parameters TYPE jsonb, ALTER COLUMN workflow_metadata TYPE jsonb"
    )
    op.execute("ALTER TABLE workflow_requests ALTER COLUMN argument TYPE jsonb")
    op.execute(
        """
    CREATE VIEW public.download_metrics AS
 SELECT ce.capability_request_id AS request_id,
    cr.capability_name AS cap_name,
    ce.created_at AS execution_created,
    ce.state,
    (cv.parameters ->> 'product_locator'::text) AS product_locator,
    sp.filegroup_id,
    (
        CASE
            WHEN ((f.datasize IS NOT NULL) AND (f.datasize <> '-1'::bigint)) THEN (f.datasize)::numeric
            WHEN (sum(f2.datasize) IS NOT NULL) THEN sum(f2.datasize)
            ELSE sum(f3.filesize)
        END)::bigint AS datasize
   FROM ((((((public.capability_executions ce
     JOIN public.capability_requests cr ON ((cr.capability_request_id = ce.capability_request_id)))
     JOIN public.capability_versions cv ON ((cr.capability_request_id = cv.capability_request_id)))
     JOIN public.science_products sp ON (((cv.parameters ->> 'product_locator'::text) = (sp.science_product_locator)::text)))
     LEFT JOIN public.filegroups f ON ((sp.filegroup_id = f.filegroup_id)))
     LEFT JOIN public.filegroups f2 ON ((sp.filegroup_id = f2.parent_filegroup_id)))
     LEFT JOIN public.files f3 ON ((sp.filegroup_id = f3.filegroup)))
  WHERE (((cr.capability_name)::text = 'test_download'::text) AND (cv.version_number = ( SELECT max(capability_versions.version_number) AS max
           FROM public.capability_versions
          WHERE (cr.capability_request_id = capability_versions.capability_request_id))))
  GROUP BY ce.capability_request_id, cr.capability_name, ce.created_at, ce.state, (cv.parameters ->> 'product_locator'::text), sp.filegroup_id, f.datasize
  """
    )


def downgrade():
    op.execute("drop view download_metrics")
    op.execute("ALTER TABLE capability_state_machines ALTER COLUMN associated_workflows TYPE json")
    op.execute(
        "ALTER TABLE capability_versions ALTER COLUMN parameters TYPE json, ALTER COLUMN workflow_metadata TYPE json"
    )
    op.execute("ALTER TABLE workflow_requests ALTER COLUMN argument TYPE json")
    op.execute(
        """CREATE VIEW public.download_metrics AS
 SELECT ce.capability_request_id AS request_id,
    cr.capability_name AS cap_name,
    ce.created_at AS execution_created,
    ce.state,
    (cv.parameters ->> 'product_locator'::text) AS product_locator,
    sp.filegroup_id,
    (
        CASE
            WHEN ((f.datasize IS NOT NULL) AND (f.datasize <> '-1'::bigint)) THEN (f.datasize)::numeric
            WHEN (sum(f2.datasize) IS NOT NULL) THEN sum(f2.datasize)
            ELSE sum(f3.filesize)
        END)::bigint AS datasize
   FROM ((((((public.capability_executions ce
     JOIN public.capability_requests cr ON ((cr.capability_request_id = ce.capability_request_id)))
     JOIN public.capability_versions cv ON ((cr.capability_request_id = cv.capability_request_id)))
     JOIN public.science_products sp ON (((cv.parameters ->> 'product_locator'::text) = (sp.science_product_locator)::text)))
     LEFT JOIN public.filegroups f ON ((sp.filegroup_id = f.filegroup_id)))
     LEFT JOIN public.filegroups f2 ON ((sp.filegroup_id = f2.parent_filegroup_id)))
     LEFT JOIN public.files f3 ON ((sp.filegroup_id = f3.filegroup)))
  WHERE (((cr.capability_name)::text = 'test_download'::text) AND (cv.version_number = ( SELECT max(capability_versions.version_number) AS max
           FROM public.capability_versions
          WHERE (cr.capability_request_id = capability_versions.capability_request_id))))
  GROUP BY ce.capability_request_id, cr.capability_name, ce.created_at, ce.state, (cv.parameters ->> 'product_locator'::text), sp.filegroup_id, f.datasize
  """
    )
